'use strict';

/* eslint no-underscore-dangle: 0 */
/* eslint no-unneeded-ternary: 0 */

var fs = require('fs');
var path = require('path');

String.prototype.lengthInUtf8 = function lengthInUtf8() {
  var asciiLength = this.match(/[\u0000-\u007f]/g) ? this.match(/[\u0000-\u007f]/g).length : 0;
  var multiByteLength = encodeURI(this.replace(/[\u0000-\u007f]/g)).match(/%/g) ? encodeURI(this.replace(/[\u0000-\u007f]/g, '')).match(/%/g).length : 0;
  return asciiLength + multiByteLength;
};

function rimraf(dirPath) {
  if (fs.existsSync(dirPath)) {
    fs.readdirSync(dirPath).forEach(function (entry) {
      var entryPath = path.join(dirPath, entry);
      if (fs.lstatSync(entryPath).isDirectory()) {
        rimraf(entryPath);
      } else {
        fs.unlinkSync(entryPath);
      }
    });
    fs.rmdirSync(dirPath);
  }
}

function uuid() {
  return Math.random().toString(26).slice(2);
}

function getMemoryStats(store) {
  var values = Array.from(store.values());
  var valuesString = JSON.stringify(values);
  return {
    size: valuesString.lengthInUtf8()
  };
}

function newDatabaseFile(filePath, fileName) {
  return new Promise(function (resolve, reject) {
    var filePathChunks = filePath.split('/');

    var breadcrumb = '';
    for (var i = 0; i < filePathChunks.length; i += 1) {
      if (filePathChunks[i]) {
        breadcrumb += '/' + filePathChunks[i];
        if (!fs.existsSync(breadcrumb)) {
          fs.mkdirSync(breadcrumb);
        }
      }
    }

    fs.writeFile(filePath + '/' + fileName + '.json', JSON.stringify({}), 'utf8', function (err) {
      if (err) {
        return reject(err);
      }
      return resolve();
    });
  });
}

function readDatabaseFile(filePath, fileName) {
  return new Promise(function (resolve, reject) {
    fs.readFile(filePath + '/' + fileName + '.json', 'utf8', function (err, data) {
      if (err) {
        return reject(err);
      }
      return resolve(data);
    });
  });
}

function writeDatabaseFile(filePath, fileName, content) {
  return new Promise(function (resolve, reject) {
    if (filePath && fileName && fs.existsSync(filePath + '/' + fileName + '.json')) {
      fs.writeFile(filePath + '/' + fileName + '.json', JSON.stringify(content), 'utf8', function (err) {
        if (err) {
          return reject(err);
        }
        return resolve(JSON.stringify(content));
      });
    } else {
      reject({ message: 'No file write' });
    }
  }).then(null, function () {});
}

function readDatabaseStats(store, filePath, fileName) {
  return new Promise(function (resolve, reject) {
    if (filePath && fileName) {
      fs.open(filePath + '/' + fileName + '.json', 'r', function (err, fd) {
        if (err) {
          return reject(err);
        }

        if (fd) {
          fs.fstat(fd, function (_err, stats) {
            if (_err) {
              return reject(_err);
            }
            return resolve(stats);
          });
        }
        return reject('No file descriptor found');
      });
    }
    return resolve(getMemoryStats(store));
  });
}

function readDatabaseStatsSync(store, filePath, fileName) {
  if (!filePath || !fileName) {
    return getMemoryStats(store);
  }

  var fd = fs.openSync(filePath + '/' + fileName + '.json', 'r');
  return fs.fstatSync(fd);
}

function resolveData(object) {
  var id = object ? object._id : null;
  if (id === undefined || id === null) {
    id = uuid();
  }
  var data = object ? object : {};
  data._id = id;
  data.created_at = new Date();
  this.store.set(id, data);

  writeDatabaseFile(this.filePath, this.fileName, Array.from(this.store.values()));
  return this.store.get(id);
}

function isObject(item) {
  return !!item && item.constructor === Object;
}

function isArray(item) {
  return !!item && item.constructor === Array;
}

module.exports = {
  rimraf: rimraf,
  uuid: uuid,
  getMemoryStats: getMemoryStats,
  newDatabaseFile: newDatabaseFile,
  readDatabaseFile: readDatabaseFile,
  writeDatabaseFile: writeDatabaseFile,
  readDatabaseStats: readDatabaseStats,
  readDatabaseStatsSync: readDatabaseStatsSync,
  resolveData: resolveData,
  isObject: isObject,
  isArray: isArray
};