'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/* eslint no-underscore-dangle: 0 */
/* eslint no-unneeded-ternary: 0 */

var fs = require('fs');
var path = require('path');
var EventEmitter = require('events');

var ModokEmitter = new EventEmitter();

/* Helper */

var _require = require('./helper'),
    newDatabaseFile = _require.newDatabaseFile,
    readDatabaseFile = _require.readDatabaseFile,
    readDatabaseStats = _require.readDatabaseStats,
    readDatabaseStatsSync = _require.readDatabaseStatsSync,
    resolveData = _require.resolveData,
    isObject = _require.isObject,
    isArray = _require.isArray;

/* Core */

var Brainstack = {};

var Brain = function Brain(name, config) {
  var _this = this;

  if (config && isObject(config)) {
    this.filePath = typeof config.filepath === 'string' ? path.resolve(config.filepath) : null;
    this.fileName = config.filename ? config.filename : name;
    this.writeToFile = fs && isObject(fs) && !this.filePath;

    if (this.filePath && this.fileName) {
      if (!fs.existsSync(this.filePath + '/' + this.fileName + '.json')) {
        newDatabaseFile(this.filePath, this.fileName).then(function () {
          ModokEmitter.emit('ready');
        });
      } else {
        readDatabaseFile(this.filePath, this.fileName).then(function (content) {
          _this.$insertMany(JSON.parse(content)).then(function () {
            ModokEmitter.emit('ready');
          });
        });
      }
    } else {
      ModokEmitter.emit('ready');
    }
  } else {
    ModokEmitter.emit('ready');
  }

  if (!name) {
    throw Error('Name is required');
  }

  if (!this) {
    if (!Brainstack[name]) {
      throw Error('No database with name ' + name + ' found');
    } else {
      return Brainstack[name];
    }
  } else {
    this.store = new Map();
    this.name = name;
    Brainstack[name] = this;
  }

  return this;
};

/**
 * ready
 * Returns a promise.
 */

Brain.prototype.ready = function ready() {
  return new Promise(function (resolve) {
    ModokEmitter.on('ready', function () {
      return resolve();
    });
  });
}();

/**
 * has / $has
 * Returns a boolean asserting whether a value has been
 * associated to the index in the database or not.
 */

// Synchronous
Brain.prototype.has = function has(_id) {
  return this.store.has(_id);
};

// Asynchronous
Brain.prototype.$has = function $has(_id) {
  var _this2 = this;

  return new Promise(function (resolve) {
    resolve(_this2.store.has(_id));
  });
};

/**
 * insert / $insert
 * Inserts a document or documents into a collection.
 * _id: If the document does not specify an _id field,
 * then modokDB will add the _id field and assign a
 * unique id for the document before inserting.
 * If the document contains an _id field, the _id value
 * must be unique within the collection to avoid overwrites.
 */

// Synchronous
Brain.prototype.insert = function insert(object) {
  if (object && Array.isArray(object)) {
    var list = [];
    for (var i = 0; i < object.length; i += 1) {
      list.push(resolveData.call(this, object[i]));
    }
    return list;
  }
  return resolveData.call(this, object);
};

// Asynchronous
Brain.prototype.$insert = function $insert(object) {
  var _this3 = this;

  return new Promise(function (resolve) {
    resolve(_this3.insert(object));
  });
};

/**
 * insertOne / $insertOne
 * Inserts a document into a collection.
 * _id: If the document does not specify an _id field,
 * then modokDB will add the _id field and assign a
 * unique id for the document before inserting.
 * If the document contains an _id field, the _id value
 * must be unique within the collection to avoid overwrites.
 */

// Synchronous
Brain.prototype.insertOne = function insertOne(object) {
  if (isObject(object)) {
    return this.insert(object);
  }

  return null;
};

// Asynchronous
Brain.prototype.$insertOne = function $insertOne(object) {
  var _this4 = this;

  return new Promise(function (resolve) {
    if (isObject(object)) {
      return resolve(_this4.insert(object));
    }
    return resolve(null);
  });
};

/**
 * insertMany / $insertMany
 * Inserts multiple documents into a collection.
 * _id: If the document does not specify an _id field,
 * then modokDB will add the _id field and assign a
 * unique id for the document before inserting.
 * If the document contains an _id field, the _id value
 * must be unique within the collection to avoid overwrites.
 */

// Synchronous
Brain.prototype.insertMany = function insertMany(object) {
  if (isArray(object)) {
    return this.insert(object);
  }

  return null;
};

// Asynchronous
Brain.prototype.$insertMany = function $insertMany(object) {
  var _this5 = this;

  return new Promise(function (resolve) {
    if (isArray(object)) {
      return resolve(_this5.insert(object));
    }
    return resolve(null);
  });
};

/**
 * find / $find
 * Selects documents in a collection or view and returns a cursor to the selected documents.
 */

// Synchronous
Brain.prototype.find = function find(object) {
  if (isObject(object)) {
    var keys = Object.keys(object);
    var values = Object.values(object);
    var foundById = null;
    var countById = 0;

    if (object && object._id !== undefined) {
      var document = this.store.get(object._id);

      if (document) {
        for (var i = 0; i < keys.length; i += 1) {
          if (document[keys[i]] === values[i]) {
            countById += 1;
          }
        }

        if (countById === keys.length) {
          foundById = document;
        }
      }

      return foundById;
    }

    var found = [];
    this.store.forEach(function (value) {
      var count = 0;

      for (var _i = 0; _i < keys.length; _i += 1) {
        if (value[keys[_i]] === values[_i]) {
          count += 1;
        }
      }

      if (count === keys.length) {
        found.push(value);
      }
    }, this.store);
    return found && found.length > 0 ? found : null;
  } else if (object === undefined) {
    return Array.from(this.store.values());
  }

  return null;
};

// Asynchronous
Brain.prototype.$find = function $find(object) {
  var _this6 = this;

  return new Promise(function (resolve) {
    resolve(_this6.find(object));
  });
};

/**
 * findOne / $findOne
 * Returns one document that satisfies the specified query criteria on the collection or view.
 * If multiple documents satisfy the query, this method returns the first document according to
 * the natural order which reflects the order of documents on the disk. In capped collections,
 * natural order is the same as insertion order. If no document satisfies the query,
 * the method returns null.
 */

// Synchronous
Brain.prototype.findOne = function findOne(object) {
  if (isObject(object)) {
    var keys = Object.keys(object);
    var values = Object.values(object);
    var foundById = null;
    var countById = 0;

    if (object._id) {
      var document = this.store.get(object._id);

      if (document) {
        for (var i = 0; i < keys.length; i += 1) {
          if (document[keys[i]] === values[i]) {
            countById += 1;
          }
        }

        if (countById === keys.length) {
          foundById = document;
        }
      }

      return foundById;
    }

    var found = [];
    this.store.forEach(function (value) {
      var count = 0;

      for (var _i2 = 0; _i2 < keys.length; _i2 += 1) {
        if (value[keys[_i2]] === values[_i2]) {
          count += 1;
        }
      }

      if (count === keys.length && found.length < 1) {
        found.push(value);
      }
    }, this.store);

    return found && found.length > 0 ? found[0] : {};
  } else if (object === undefined) {
    return Array.from(this.store.values())[0] || null;
  }

  return null;
};

// Asynchronous
Brain.prototype.$findOne = function $findOne(object) {
  var _this7 = this;

  return new Promise(function (resolve) {
    resolve(_this7.findOne(object));
  });
};

/**
 * update / $update
 * Modifies an existing document or documents in a collection.
 * The method can modify specific fields of an existing document or
 * documents or replace an existing document entirely, depending on the update parameter.
 */

// Synchronous
Brain.prototype.update = function update(_query, _update, _config) {
  var MergedDocuments = [];
  var documents = this.find(_query);
  if (documents && isArray(documents) && documents.length > 0) {
    for (var i = 0; i < documents.length; i += 1) {
      MergedDocuments.push(Object.assign(documents[i], _update));
    }
    return this.insert(MergedDocuments);
  } else if (documents && isObject(documents)) {
    return this.insert(Object.assign(documents, _update));
  } else if (_config && isObject(_config) && _config.upsert === true) {
    return this.insert(Object.assign(_query, _update));
  }
  return null;
};

// Asynchronous
Brain.prototype.$update = function $update(_query, _update, _config) {
  var _this8 = this;

  return new Promise(function (resolve) {
    _this8.$find(_query).then(function (documents) {
      if (documents && isArray(documents) && documents.length > 0) {
        var MergedDocuments = [];
        for (var i = 0; i < documents.length; i += 1) {
          MergedDocuments.push(Object.assign(documents[i], _update));
        }
        resolve(_this8.$insert(MergedDocuments));
      } else if (documents && isObject(documents)) {
        resolve(_this8.$insert(Object.assign(documents, _update)));
      } else if (_config && isObject(_config) && _config.upsert === true) {
        resolve(_this8.$insert(Object.assign(_query, _update)));
      }
    });
  });
};

/**
 * updateOne / $updateOne
 * Modifies a single existing document in a collection.
 * The method can modify specific fields of an existing document or
 * replace an existing document entirely, depending on the update parameter.
 */

// Synchronous
Brain.prototype.updateOne = function updateOne(_query, _update, _config) {
  var documents = this.find(_query);
  if (documents && isArray(documents) && documents.length > 0) {
    return this.insert(Object.assign(documents[0], _update));
  } else if (documents && isObject(documents)) {
    return this.insert(Object.assign(documents, _update));
  } else if (_config && isObject(_config) && _config.upsert === true) {
    return this.insert(Object.assign(_query, _update));
  }
  return null;
};

// Asynchronous
Brain.prototype.$updateOne = function $updateOne(_query, _update, _config) {
  var _this9 = this;

  return new Promise(function (resolve) {
    _this9.$find(_query).then(function (documents) {
      if (documents && isArray(documents) && documents.length > 0) {
        return resolve(_this9.$insert(Object.assign(documents[0], _update)));
      } else if (documents && isObject(documents)) {
        return resolve(_this9.$insert(Object.assign(documents, _update)));
      } else if (_config && isObject(_config) && _config.upsert === true) {
        return resolve(_this9.$insert(Object.assign(_query, _update)));
      }
      return resolve(null);
    });
  });
};

/**
 * updateMany / $updateMany
 * Modifies an existing document or documents in a collection.
 * The method can modify specific fields of an existing document or
 * documents or replace an existing document entirely, depending on the update parameter.
 */

// Synchronous
Brain.prototype.updateMany = Brain.prototype.update;

// Asynchronous
Brain.prototype.$updateMany = Brain.prototype.$update;

// Synchronous
Brain.prototype.count = function count() {
  return this.store.size;
};

// Asynchronous
Brain.prototype.$count = function $count() {
  var _this10 = this;

  return new Promise(function (resolve) {
    resolve(_this10.store.size);
  });
};

/**
 * drop / $drop
 * Removes a collection from the database.
 */

// Synchronous
Brain.prototype.drop = function drop() {
  this.store.clear();
  return this.store;
};

// Asynchronous
Brain.prototype.$drop = function $drop() {
  var _this11 = this;

  return new Promise(function (resolve) {
    _this11.store.clear();
    resolve(_this11.store);
  });
};

/**
 * stats / $stats
 * Returns statistics about the collection.
 */

// Synchronous
Brain.prototype.stats = function stats() {
  var store = this.store;
  return readDatabaseStatsSync(store, this.filePath, this.fileName);
};

// Asynchronous
Brain.prototype.$stats = function $stats() {
  var _this12 = this;

  var store = this.store;
  return new Promise(function (resolve) {
    resolve(readDatabaseStats(store, _this12.filePath, _this12.fileName));
  });
};

/**
 * renameCollection / $renameCollection
 * Renames a collection.
 */

// Synchronous
Brain.prototype.renameCollection = function renameCollection(name) {
  delete Brainstack[this.name];
  this.name = name;
  Brainstack[name] = this;
  return this;
};

// Asynchronous
Brain.prototype.$renameCollection = function $renameCollection(name) {
  var that = this;
  return new Promise(function (resolve) {
    delete Brainstack[that.name];
    that.name = name;
    Brainstack[name] = that;
    resolve(that);
  });
};

/**
 * deleteOne / $deleteOne
 * Deletes the first document that matches the query.
 * Use a field that is part of a unique index such as _id for precise deletions.
 */

// Synchronous
Brain.prototype.deleteOne = function deleteOne(object) {
  if (isObject(object)) {
    var keys = Object.keys(object);
    var values = Object.values(object);
    var foundById = null;
    var countById = 0;

    if (object._id) {
      var document = this.store.get(object._id);

      if (document) {
        for (var i = 0; i < keys.length; i += 1) {
          if (document[keys[i]] === values[i]) {
            countById += 1;
          }
        }

        if (countById === keys.length) {
          foundById = document;
        }
      }

      if (foundById && foundById._id) {
        return this.store.delete(foundById._id);
      }

      return false;
    }

    var found = [];
    this.store.forEach(function (value) {
      var count = 0;

      for (var _i3 = 0; _i3 < keys.length; _i3 += 1) {
        if (value[keys[_i3]] === values[_i3]) {
          count += 1;
        }
      }

      if (count === keys.length && found.length < 1) {
        found.push(value);
      }
    }, this.store);

    if (found && found.length > 0) {
      return this.store.delete(found[0]._id);
    }

    return false;
  }

  return false;
};

// Asynchronous
Brain.prototype.$deleteOne = function $deleteOne(object) {
  var _this13 = this;

  return new Promise(function (resolve) {
    var value = _this13.deleteOne(object);
    resolve(value);
  });
};

/**
 * Removes all documents that match the filter from a collection.
 */
Brain.prototype.deleteMany = function deleteMany(object) {
  if (isObject(object)) {
    var keys = Object.keys(object);
    var values = Object.values(object);
    if (object._id) {
      return this.deleteOne(object);
    }

    var found = [];
    this.store.forEach(function (value) {
      for (var i = 0; i < keys.length; i += 1) {
        if (value[keys[i]] === values[i]) {
          found.push(value);
        }
      }
    }, this.store);

    if (found && found.length > 0) {
      for (var i = 0; i < found.length; i += 1) {
        this.store.delete(found[i]._id);
      }
      return true;
    }

    return false;
  }

  return false;
};

Brain.prototype.$deleteMany = function $deleteMany(object) {
  var _this14 = this;

  return new Promise(function (resolve) {
    resolve(_this14.deleteMany(object));
  });
};

Brain.prototype.delete = Brain.prototype.deleteMany;
Brain.prototype.$delete = Brain.prototype.$deleteMany;

Brain.prototype.deleteOneById = function deleteOne(index) {
  return this.store.delete(index);
};

Brain.prototype.$deleteOneById = function $deleteOne(index) {
  var _this15 = this;

  return new Promise(function (resolve) {
    resolve(_this15.deleteOneById(index));
  });
};

// TODO //

/*

Brain.prototype.findAndModify = function findAndModify(object) {
  return object;
};

Brain.prototype.findOneAndDelete = function findOneAndDelete(object) {
  return object;
};

Brain.prototype.findOneAndReplace = function findOneAndReplace(object) {
  return object;
};

Brain.prototype.findOneAndUpdate = function findOneAndUpdate(object) {
  return object;
};

Brain.prototype.getIndexes = function getIndexes() {
  return Array.from(this.store.keys());
};
*/

var main = Brain;

exports.default = main;

module.exports = main; // for CommonJS compatibility